@@ -7,10 +7,13 @@ utils::globalVariables(
     "est",
     "lci",
     "lsm_type",
+    "n_miss",
     "original_id",
     "parameter",
     "parameter_type",
+    "pct_miss",
     "pval",
+    "remainder",
     "se",
     "uci",
     "visit",
